;Sprink06.asm -- Water Sprinkler Timer

;Modified old Timer3.asm code to follow scheme used for Kitchen Timer 4/29/06
;That is hold down counts add one hour, dahs add 10 minutes, and dits add one minute.

;Current draw at idle = 
;Short Dit Touch = one minute addition
;Longer Dah Touch = 10 minute additions
;Hold down count - one hour addition
;Internal oscillator in ATTINY2313 is running micro-controller at approx. 1MHz

;September 26, 2006
;June 18, 2007 

.include "2313def.inc"

rjmp	START1
rjmp	START1
rjmp	START1
rjmp	START1
rjmp	ClockCntDwnInt		;one second interval interrupt
rjmp	START1
rjmp	START1

	
START1:				;set up stack pointer and initialize cold start

ldi		r16,low(RAMEND)		; Load low byte address of end of RAM into register R16
out		SPL,R16				; Initialize stack pointer to end of internal RAM

ser		r16			;Set to all ones for out port direction as output
out		DDRB,r16	;Set PORTB to output
out		PortB,r16	;LEDs are off when port pins are high

ldi		R16,120		;Set bits 0, 1, & 2 as inputs ... remaining bits as output
out		DDRD,R16	;execute data direction with bit 0 as input
out		PORTD,R16	;execute to set ports high, with no input resistor

in		r16,TIMSK	;enable 16-bit match to A register
ori		r16,64		
out		TIMSK,r16

in		r16,TCCR1B
ori		r16,11		; divide by 64 on prescaler
out		TCCR1B,r16	;enable reset of counter to zero (bit3) after match detection.
					;and bits 0,1,2 select prescaling source as 64 of clock

ldi		r16,218
out		OCR1AH,r16
ldi		r16,140		;default selected for one second interval interrupt (70 deg)
out		OCR1AL,r16	;loads the compare register A for timer 1 (1 second)
					;set for 3,59 MHz clock frequency


InitializeClkTime:		;starts clock at 8:00 a.m.

ldi		r16,1			;put Sunday(1) in as start of day-of-week
mov		r14,r16			; put into r14, day of week counter

ldi		R30,224			;8:00 A.M. low byte
ldi		r31,1			;8:00 A.M. high byte

inc		r15				;put one second into seconds counter

ldi		r16,10
sts		$61,r16			;default load of clock correction seconds
	
ldi		r16,11
sts		$62,r16			; $62 =permanent store for interval, 
						; default = 11 = (every 11th day) 
mov		r9,r16			; R96 =temporary store for $62 above, 
						; 0= no correction, 1 = every day
						; 2 = every other day, 3 = every 3rd day, etc.
						; default = add 1 second every 4 days

ldi		r16,1			;set How Often Counter to every day
sts		$63,r16
mov		r8,r16			;put ONE day (today) into How Often Counter variable r8

ldi		r16,234			;put default alarm start time
sts		$7C,r16			;   of 8:10 a.m. into memory
ldi		r16,1
sts		$7D,r16			

clr		r16
sts		$66,r16
sts		$67,r16
sts		$7E,r16
sts		$7F,r16
sts		$80,r16

clr		r18			;clear all flags

bset	7			;enable global interrupts

rcall	DelayOne		;allow circuit to settle
rcall	AllValvesOff

;-----------------------------


MainLoop:

mov		r16,r15		;get seconds counter
cpi		r16,1		
breq	ML1
cpi		r16,31
breq	ML1a
rjmp	ML2


ML1:

rcall	ITenWag


ML1a:

rcall IDisplayClock


ML2:

sbis	PIND,0		;skip if D0 set (program switch up)
rcall	Decode

sbrs	r18,1		;skip if minute flag #2 set, r18,1(2)
rjmp	MainLoop

cbr		r18,2		;clear minute flag #2

sbrs	r18,3		;skip if flood flag set, r18,3(8) (there's a flood)
rjmp	ML4


ML3:				;on flood flag then off valves & pending

rcall	AllValvesOff

lds		r16,$80		;retrieve original Execute Series routing number
sts		$7F,r16		; and put in routing number location.

rcall	AllValvesOff
clr		r16
sts		$66,r16		;zero number of commands alarm
sts		$67,r16		;zero number of commands immediate

rjmp	MainLoop


ML4:

sbis	PIND,1		;skip if rain detector is set (rain NOT detected)
rjmp	ML3			;rain is detected

rcall	ExecuteSeries
rjmp	MainLoop

;----------------------------------


Decode:				;Monitor Button push and decode

push	r17
push	r20
push	r21
push	r23				;save registers used during Button6 & Decode
						;THESE registers popped in DecodeEnd
	
rcall	ButtonPush6		;check for entry

cpi		r20,0			;checks to see if LONG Count has been recorded
breq	TPShortCount	;branch if same or higher than 1
rjmp	HCStart


TPShortCount:		;decode short counts


TPSC1:

cpi		r21,1		;Togle valve 1 on/off
brne	TPSC2

sbis	PortB,3		;skip if Valve #1 off
rjmp	TPSC1a

ldi		r16,1		;select which valve
rcall	OnSelectedValve
rjmp	DecodeEnd


TPSC1a:

rcall	AllValvesOff

rjmp	DecodeEndR	;restore flag status

;-------------


TPSC2:

cpi		r21,2		;Toggle valve #2 on/off
brne	TPSC3

sbis	PortB,4		;skip if valve off
rjmp	TPSC2a

ldi		r16,2		;select valve #2
rcall	OnSelectedValve
rjmp	DecodeEnd


TPSC2a:

rcall	AllValvesOff
rjmp	DecodeEndR

;----------------------


TPSC3:

cpi		r21,3		;Toggle valve#3  on/off
brne	TPSC4

sbis	PortB,5		;skip if valve off
rjmp	TPSC3a

ldi		r16,3		;select which valve
rcall	OnSelectedValve
rjmp	DecodeEnd


TPSC3a:

rcall	AllValvesOff
rjmp	DecodeEndR

;---------------------


TPSC4:				;toggle valve #4 on/off

cpi		r21,4
brne	TPSC5

sbis	PortB,6		;skip if valve off
rjmp	TPSC4a

ldi		r16,4		;select which valve
rcall	OnSelectedValve
rjmp	DecodeEnd


TPSC4a:

rcall	AllValvesOff
rjmp	DecodeEndR

;---------------------


TPSC5:				;toggle valve #5 on/off

cpi		r21,5
brne	TPSC6

sbis	PortB,7		;skip if valve off
rjmp	TPSC5a

ldi		r16,5		;select which valve
rcall	OnSelectedValve
rjmp	DecodeEnd


TPSC5a:

rcall	AllValvesOff
rjmp	DecodeEndR

;-------------------------


TPSC6:				;off all valves - clear all valve flags

cpi		r21,6		;off all valves for any number 6 and above
brsh	TPSC6a
rjmp	DecodeEnd


TPSC6a:

lds		r16,$80		;retrieve original Execute Series routing number
sts		$7F,r16		; and put in routing number location.

rcall	AllValvesOff
clr		r16
sts		$66,r16		;zero number of commands alarm
sts		$67,r16		;zero number of commands immediate
cbr		r18,8		;clear flood flag, r18,3(8)
sbi		PortB,2		;set next LED off, PortB,2(8)
sbi		PortB,0		;set PM LED off, PortB,0(1)

rjmp	DecodeEndR

;-------------------------------------


TPHoldDownCount:		;series for long counts

;-----------------------

HCStart:

HC1:				;Load commands for Immediate On series

cpi		r20,1		;Enter a series of immediate on commands
breq	HC1a
rjmp	HC2


HC1a:

push	r7
push	r28
clr		r7			;zero number of commands counter
ldi		r28,$72		;start of immediate commands memory section
sts		$81,r28		;store starting pointer location


HC1aa:

ldi		r16,2
rcall	SlowGreenFlash	;signal two entries are needed
rcall	ButtonStall
cpi		r20,0			;HC value of zero EXITS
breq	HC1OnValve


HC1b:

cpi		r20,6
brsh	HC1onValve
mov		r22,r20		;store valve number temporarily in r22

rcall	OneGreenFlash	;signal need for next entry (dit-dah entries)

rcall	ButtonStall
st		y+,r21		;store time on (dit=1, dah=10 added time count)
st		y+,r22		;store valve number
sts		$81,r28		;store present pointer location

inc		r7			;incremenet command number
sts		$67,r7		;store number of commands in $67

cpi		r28,$7C		;check for end of memory
brsh	HC1onValve

rjmp	HC1aa		;jump back for new entry


HC1onValve:

tst		r7			;test for entry.  Zero equals no entry
brne	HC1d
rjmp	HC1End


HC1d:

rcall	AllValvesOff

ldi		r16,1		;set immediate on flag = 1
sts		$7F,r16

ldi		r28,$72		;load start of memory for immediate commands
ld		r11,y+		;load time on and post increment
ld		r16,y+		;get valve number and post increment
sts		$81,r28		;store pointer location in $81

rcall	OnSelectedValve	;turn on the valve number in r16.


HC1End:

pop		r28
pop		r7

rjmp	DecodeEnd

;---------------------------


HC2:				;off valves and halt any series

cpi		r20,2
breq	HC2a
rjmp	HC3


HC2a:

lds		r16,$80		;retrieve original Execute Series routing number
sts		$7F,r16		; and put in routing number location.

rcall	AllValvesOff
clr		r16
sts		$66,r16		;zero number of commands alarm
sts		$67,r16		;zero number of commands immediate
cbr		r18,8		;clear flood flag, r18,3(8)
sbi		PortB,2		;set next LED off, PortB,2(8)
sbi		PortB,0		;set PM LED off, BortB,0(1)

rjmp	DecodeEnd

;---------------------------


HC3:				;Turns on the Alarm Time valve series immediately

cpi		r20,3
breq	HC3a
rjmp	HC4


HC3a:

push	r28

ldi		r16,3
sts		$7F,r16		;set rounting in temp to continue alarm series

lds		r16,$7E		;load number of commands
dec		r16			;decrement number of commands
sts		$66,r16		;store back in temp $66

ldi		r28,$68		;set up pointer to start of memory store
ld		r10,y+		;load time on in r10 and post increment
ld		r16,y+		;load valve number on and post increment
sts		$81,r28		;store r28 into pointer store

rcall	OnSelectedValve	;on valve number stored in r16

pop		r28

rjmp	DecodeEnd

;-----------------------


HC4:					;Toggle alarm series on or off

cpi		r20,4
breq	HC4a
rjmp	HC5


HC4a:

sbrs	r18,2			;skip if alarm on bit set, r18,2(4)
rjmp	HC4b

clr		r16
sts		$80,r16			;put off alarm in former Execute routing
sts		$7F,r16			;put off alarm in temporary Execute routing
cbr		r18,4			;off alarm flag, r18,2(4)

ldi		r16,1
rcall	QuickRedFlash		;short red flash
rcall	DelayHalf

rjmp	DecodeEnd


HC4b:

ldi		r16,2
sts		$80,r16			;put ON alarm in former Execute routing
sts		$7F,r16			;put ON alarm in temp Execute routing
sbr		r18,4			;set alarm on bit, r18,2(4)

cbi		PortB,0			;on red LED
rcall	DelayOne
sbi		PortB,0			;off red LED (for long flash)
rcall	DelayHalf
sbr		R18,4			;on alarm flag, r18,2(4)

rjmp	DecodeEnd

;-----------------------------------


HC5:			;Set alarm time for valve series to start

cpi		r20,5
breq	HC5aa
rjmp	HC6


HC5aa:

push	r0
push	r1
push	r24
push	r28
push	r29

clr		r0			;clear scratch registers
clr		r1


HC5a:

ldi		r16,4
rcall	SlowGreenFlash	;flashes indicate four entries needed
rcall	ButtonStall		;enter MSD hours (24 hour format)
rcall	Mult10			;multiply r20 value by 10
mov 	r24,r20			;store in r0

rcall	OneGreenFlash	;indicating one more entry for LSD hours
rcall	ButtonStall
add		r24,r20			;add LSD of hours


HC5b:

tst		r24				;entering total hours 
breq	HC5d
ldi		r16,60
add		r0,r16			;adds 60 minutes for each iteration
brcc	HC5bb
inc		r1


HC5bb:

dec		r24				;decrement r24
brne	HC5b			


HC5d:

ldi		r16,2
rcall	SlowGreenFlash	;flash twice indicating two entries needed
rcall	ButtonStall		;  MSD minutes entry
rcall	Mult10			;multiply r20 by 10

add		r0,r20			;add MSD of minutes
brcc	HC5dd
inc		r1


HC5dd:

rcall	OneGreenFlash	;indicating one more entry for LSD minutes
rcall	ButtonStall
add		r0,r20			;add LSD of minutes
brcc	HC5ddd
inc		r1


HC5ddd:

sts		$7C,r0
sts		$7D,r1			;put completed time into SRAM

pop		r29
pop		r28
pop		r24
pop		r1
pop		r0

rjmp	DecodeEnd

;---------------------------------


HC6:				;set every other day interval in perm store

cpi		r20,6
breq	HC6a
rjmp	HC7


HC6a:

rcall	OneGreenFlash	;indicates one entry needed
rcall	ButtonStall
sts		$63,r20
mov		r8,r20			;put same number into How Often variable

rjmp	DecodeEnd

;--------------------------------


HC7:			;Set temporary days away for interval

cpi		r20,7
breq	HC7a
rjmp	HC8


HC7a:

rcall	OneGreenFlash	;indicates one entry needed
rcall	ButtonStall
mov		r8,r20

rjmp	DecodeEnd

;-----------------------------------


HC8:				;Program Valves and Times for Alarm Time series

cpi		r20,8
breq	HC8a
rjmp	HC10


HC8a:

push	r5
push	r17
push	r28

clr		r5			;clear number of commands count, r5
ldi		r28,$68		;start of alarm commands memory section


HC8aa:

ldi		r16,2
rcall	SlowGreenFlash	;signal two entries are needed
rcall	ButtonStall
cpi		r20,1		;valve number 1 to 5 (zero exits)
brsh	HC8b
rjmp	HC8End


HC8b:

cpi		r20,6
brsh	HC8End		;number is too high - exit

rcall	OneGreenFlash	;signal need for next entry (dit-dah)

mov		r17,r20		;store valve number temporarily

rcall	ButtonStall
st		y+,r21		;store time on from r21
st		y+,r17		;store valve number in r16
inc		r5			;incremenet number of commands number
sts		$7E,r5		;put in perm store
sts		$66,r5		;put in temporary store

cpi		r28,$72		;check for end of memory
brsh	HC8End

rjmp	HC8aa		;jump back for new entry


HC8End:

pop		r28
pop		r17
pop		r5

rjmp	DecodeEnd

;-----------------------------


HC10:				;Set CLOCK

cpi		r20,10
breq	HC10a
rjmp	HC11


HC10a:

push	r16
rcall	SetClock
pop		r16
rjmp	DecodeEnd


HC11:						;Set LSB of crystal countdown register

cpi		r20,11
breq	HC11a
rjmp	HC12


HC11a:

push	r16

rcall	SetCompareRegister


pop		r16
rjmp	DecodeEnd


HC12:					;Clock correction at Midnight

cpi		r20,12
breq	HC12a
rjmp	DecodeEnd


HC12a:

push	r16

rcall	SecondsCorrection

pop		r16
rjmp	DecodeEnd

;------------------------------


HCEnd:


DecodeEndR:			;restores alarm on flag for only off commands

lds		r16,$80
sts		$7F,r16		;restore alarm on status
cbr		r19,1		;clear one time alarm flag


DecodeEnd:

pop		r23
pop		r21
pop		r20
pop		r17			;restore registers used during button & decode

rcall	ThreeWag	;wag LEDs three times

ret

;-------------------------------


ClockCntDwnInt: 		;interrupt to update clock counters

push	r16

in		r16,SREG	;save flags
sts		$60,r16

sbrs	r18,3		;skip if flood flag is set, r18,3(8)
rjmp	CCflood1
cbi		PortB,2		;on "next" LED - PortB,2(4)


CCflood1:

sbis	PIND,2		;skip if flood detection bit set PD2 (no flood)
rjmp	CCDIflood

sbis	PIND,1		;skip if rain detection bit set PD1 (no rain)
rjmp	CCDIrain
rjmp	CCDI1a


CCDIflood:

sbr		r18,8			;set flood flag on, r18,3(8)
rcall	AllValvesOff	;turn off all valves
rjmp	CCDI1a


CCDIrain:

rcall	AllValvesOff
cbi		PortB,1			;clear PortB,1(2) - on rain AM (green) LED


CCDI1a:

inc		r15			;step one count of SECONDS counter
mov		r16,r15
cpi		r16,60
breq	CCDI1

rjmp	CCDIEnd		;return since no action required


CCDI1:

clr		r15			;zero r15 -- SECONDS counter
sbr		r18,3		;set bit 1(2) 0(1) -- the NEW MINUTE flags 1 & 2

adiw	r30,1		;increment minutes reg pair "z" (r30,31) by one

cpi		r31,5		;compare high byte for match
brne	CCDIEnd

cpi		r30,160		;compare low byte for match
brne	CCDIEnd


CCDI1c:			;zero high and low bytes - it is midnight

clr		r30
clr		r31			;zero minutes counter

dec		r8
brne	CCDI2howoften	;branch if not equal zero

lds		r8,$63		;fetch new how often number


CCDI2howoften:

mov		r16,r14
inc		r16			;increment day of week counter
cpi		r16,8		;if >8 then change count
brne	CCDI1cc

ldi		r16,1		;change day count to 1, Sunday


CCDI1cc:

mov		r14,r16		;move correct day of week value into storage


ClockCorrection:	;adds seconds in $61 to time at minight 
					; determined by $62 & $63 (interval)

tst		r9			;if zero, no correction is needed
					;1= every day, 2= every other day, etc.

breq	CCDIEnd		;exit routine with no action

dec		r9
brne	CCDIEnd

lds		r9,$62		;reload interval
	

CC3:

lds	r16,$61		;seconds correction from permanent store
mov	r15,r16		;put in seconds store area to correct time (add)


CCDIEnd:

lds	r16,$60		;SREG temporary store
out	SREG,r16	;restore SREG data

pop	r16

reti

;------------------------


ExecuteSeries:		;Routes according to $7F routing number
					;0=nothing
					;1=Immediate On Continue
					;2=Timed alarm on
					;3=Timed alarm continue

lds		r22,$7F		;fetch routing number


ES1:			;Immediate On Continue

cpi		r22,1
breq	CFM4
rjmp	ES2


CFM4:

push	r28			;for pointer manipulation

lds		r16,$67		;fetch number of commands
cpi		r16,0
breq	CFM4End

dec		r11			;decrenent time on count	
breq	CFM4a
rjmp	CFM4End


CFM4a:

rcall	AllValvesOff

lds		r16,$67		;fetch number of commands
dec		r16
sts		$67,r16		;decrement number of commands and store
brne	CFM4c
lds		r16,$80
sts		$7F,r16		;restore flag status from $80
rjmp	CFM4End


CFM4c:

lds		r28,$81		;fetch pointer from pointer location
ld		r11,y+		;get new on time 
ld		r16,y+		;get valve number
sts		$81,r28		;store present Y pointer location in $81

rcall	OnSelectedValve


CFM4End:

pop		r28
rjmp	ESEnd


;--------------------------------

ES2:				;Timed alarm on first time through

cpi		r22,2
breq	CFM1
rjmp	ES3


CFM1:				;Alarm routing is "2" - alarm flag is set

push	r5
push	r28

mov		r16,r8		;move how often variable into r16 for check
cpi		r16,1
breq	CFM1b		;jump out if not equal
rjmp	CFM1End		;jump out


CFM1b:

lds		r16,$7D		;load MSD of time 
cp		r16,r31		;is high time byte same as clock time?
breq	CFM1bb
rjmp	CFM1End


CFM1bb:

lds		r16,$7C		;load LSD of time
cp		r16,r30		;compare against clock time
breq	CFM1c
rjmp	CFM1End


CFM1c:

ldi		r16,3
sts		$7F,r16		;change to alarm continue routing

lds		r5,$7E		;grab number of commands from perm store, $7E
dec		r5
sts		$66,r5		;store decremented in $66

ldi		r28,$68		;get start of memory location for Alarm Comm
ld		r10,y+		;get value of time on (1-255 min) and post incr
ld		r16,y+		;get valve number and post incr

rcall	OnSelectedValve	;on the valve stated in r16


CFM1End:

sts		$81,r28		;store pointer in $81

pop		r28
pop		r5

rjmp	ESEnd

;-------------------------


ES3:				;Timed alarm continue

cpi		r22,3
breq	CFM3
rjmp	ESEnd


CFM3:

push	r28

lds		r28,$81		;fetch present pointer location

tst		r10			;test alarm on minutes
brne	CFM3a
rjmp	CFM3End


CFM3a:

dec		r10
breq	CFM3b
rjmp	CFM3End


CFM3b:

rcall	AllValvesOff
lds		r16,$66		;check number of commands
cpi		r16,0
breq	CFM3c
dec		r16
sts		$66,r16		;store decremented no. of commands in $66

ld		r10,y+		;get new on time and post increment
ld		r16,y+		;get new valve number and post increment

rcall	OnSelectedValve	;r16 has value

rjmp	CFM3End


CFM3c:

lds		r16,$80		;grab former alarm on flag status
sts		$7F,r16		;set temp routing to alarm on

rjmp	CFM3End


CFM3End:

sts		$81,r28		;store present pointer location

pop		r28

rjmp	ESEnd

;------------------------------------


ESEnd:

ret

;-------------------------------


VariableDelay:	;r16 contains number of 40th second delays desired

rcall	Delay40th
dec		r16
breq	VD2
rjmp	VariableDelay


VD2:

ret

;--------------------------------------


IVariableDelay:	;r16 contains number of interruptible variable 40th delays

rcall	IDelay40th
dec		r16
breq	IVD2
rjmp	IVariableDelay


IVD2:

ret

;----------------------------------------


Delay40th:		;1/40th second delay

push	r0
push	r16
clr		r0
ldi		r16,116


D40a:

dec		r0
brne	D40a

dec		r16
brne	D40a

pop		r16
pop		r0

ret

;------------------------------------------


IDelay40th:			;interruptible 1/40th second delay

push	r0
push	r16

clr		r0
ldi		r16,69


ID40a:

sbis	PIND,0	;skip if switch is up - bit 2(4)
rcall	Decode
dec		r0
brne	ID40a

dec		r16
brne	ID40a

pop		r16
pop		r0

ret

;-----------------------------------------


DelayTenth:

push 	r16
ldi		r16,4


DelayTenth1:

rcall	Delay40th
dec		r16
brne	DelayTenth1

pop		r16
ret

;-----------------------------------------


DelayHalf:

push 	r16
ldi		r16,20

rjmp	DelayTenth1

;-------------------------------------------


DelayOne:

push	r16
ldi		r16,40

rjmp	DelayTenth1

;-------------------------------------


IDelayTenth:

push	r16
ldi		r16,4


IDelayTenth1:

rcall	IDelay40th
dec		r16
brne	IDelayTenth1

pop		r16
ret

;----------------------------


IDelayHalf:

push	r16
ldi		r16,20

rjmp	IDelayTenth1

;----------------------------------


IDelayOne:

push	r16
ldi		r16,40

rjmp	IDelayTenth1

;----------------------------------------


IThreeWag:

push	r16
ldi		r16,3

rjmp	ITenWag1

;----------------------------------------


ITenWag:				

push	r16
ldi		r16,10


ITenWag1:

cbi		PortB,0		;on red LED
rcall	IDelay40th
sbi		PortB,0		;off red LED
rcall	IDelay40th
cbi		PortB,1		;on Green LED
rcall	IDelay40th
sbi		PortB,1
rcall	IDelay40th
dec		r16
brne	ITenWag1

pop		r16
ret

;--------------------------------------------


ThreeWag:

ldi		r16,3

rjmp	Wag1	

;--------------------------------------------


TenWag:				

ldi		r16,10


Wag1:

cbi		PortB,0		;on red LED
rcall	Delay40th
sbi		PortB,0		;off red LED
rcall	Delay40th
cbi		PortB,1		;on Green LED
rcall	Delay40th
sbi		PortB,1		;off Green LED
rcall	Delay40th
dec		r16
brne	Wag1

ret

;-------------------------------------------


QuickRedFlash:		;r16 contains number of flashes

cbi		PortB,0		;on RED LED
rcall	Delay40th
sbi		PortB,0		;off RED LED
rcall	Delay40th
dec		r16
brne	QuickRedFlash

ret

;----------------------------------------


SlowRedFlash:		;r16 contains number of flashes

cbi		PortB,0		;On Red LED
rcall	Delay40th
sbi		PortB,0		;Off Red LED
rcall	DelayHalf
dec		r16
brne	SlowRedFlash

ret

;-----------------------------------------


ISlowRedFlash:			;r16 contains number of flashes

cbi		PortB,0
rcall	IDelay40th
sbi		PortB,0
rcall	IDelayHalf
dec		r16
brne	ISlowRedFlash

ret

;----------------------------------------


SlowGreenFlash:			;r16 contains number of flashes

cbi		PortB,1			;on green LED
rcall	Delay40th
sbi		PortB,1			;off green LED
rcall	DelayHalf
dec		r16
brne	SlowGreenFlash

ret

;--------------------------------------


OneGreenFlash:

ldi		r16,1
rjmp	SlowGreenFlash

;------------------------------


TwoGreenFlash:

ldi		r16,2
rjmp	SlowGreenFlash

;------------------------------


ThreeGreenFlash:

ldi		r16,3
rjmp	SlowGreenFlash

;------------------------------


IFlashNext:			;flashes next LED no of times in r16

cbi		PortB,2		;on LED
rcall	IdelayTenth
sbi		PortB,2		;off LED
rcall	IdelayHalf
dec		r16
brne	IFlashNext

ret

;---------------------------------------


ButtonPush6:		
					;r17 is code register for dit dah codes
					;r21 accumulates dit dah additions
					;r20 has HoldDown (HC) counts

sbi		PortB,0			;off all LEDs
sbi		PortB,1
sbi		PortB,2

clr		r20			;Holddown count
clr		r21			;clear dit-dah count
clr		r23			;zero universal delay counter


B1:

inc		r23			;increment universal delay counter
rcall	Delay40th
cpi		r23,70		;delay between hold down pushes
brsh	B3

sbis	PIND,0		;skip is switch is up
rjmp	B1

cpi		r23,20
brsh	B1b			;it is dah count

inc		r21			;increment r21 by one

cbi		PortB,0		;On LED briefly
rcall	Delay40th		
sbi		PortB,0		;Off LED


B1c:

clr		r23


B2d:

sbic	PIND,0		;skip if switch down
rjmp	B5

clr		r23
rjmp	B1


B1b:				;it is dah count

ldi		r16,10
add		r21,r16		;add 10 to r21 

cbi		PortB,0		;medium-short on of LED
rcall	DelayTenth
sbi		PortB,0		;off LED

rjmp	B1c


B3:					;Hold Down Count

cbi		PortB,0		;on PortB,0(1), LED for long flash of LED (HC)
rcall	DelayTenth	;1/10 second delay
sbi		PortB,0		;off LED

inc		r20			;add ONE to Holddown Counter

clr		r23			;clear universal delay counter


B4:

inc		r23
rcall	Delay40th

sbic	PIND,0		;skip if switch down
rjmp	B5

cpi		r23,30	
brsh	B3

rjmp	B4


B5:

inc		r23

rcall	Delay40th
cpi		r23,120		;3 second pause
brsh	BPEnd
rjmp	B2d


BPEnd:

ret

;-------------------------------------------


ButtonStall:			;Waits for button to be pushed & debounces

sbi		PortB,0			;off all LEDs
sbi		PortB,1
sbi		PortB,2


BS1:

clr		r16


BS3:

sbic	PIND,0		;skip if bit 0(1) -- switch is down (on)
rjmp	BS1

rcall	Delay40th
inc		r16

cpi		r16,2
brsh	BSEnd



sbis	PortB,0
rjmp	Cont1

cbi		PortB,0
rjmp	Cont2


Cont1:

sbi		PortB,0

Cont2:







rjmp	BS3


BSEnd:

rjmp	ButtonPush6		;to use return in ButtonPush6

;----------------------------------------------


IDisplayClock:		;displays time and can be interrupted in IDelays
					;Uses r24 instead of r16 to avoid conflicts 
					;  during interrupts in this routine.

cbr		r18,1		;clear seconds flag #1, r18,0(1)

push	r6
push	r16
push	r28
push	r29

clr		r6
clr		r7

rcall	ITenWag		;begin clock display with wag of LEDs

bclr	7			;turn off interrupts

mov		r28,r30		;load low byte into r28
mov		r29,r31		;load high byte into r29

bset	7			;turn interrupts back on

tst		r29			;test high byte for zero
brne	DC3

cpi		r28,60		;check to see if less than 60 (12:00 am to 12:59 am)
brlo	DC2
rjmp	DC3


DC2:

ldi		r16,12			;flash out 12 a.m.
rcall	IVarClockFlash	;flash out time of 12 a.m.

rjmp	DC7


DC3:				;flash out other times

mov		r4,r28		;save low byte in r4
mov		r5,r29		;save high byte in r5

inc		r6			;increment hours count
sbiw	r28,60		;subtract 60 from the r28,29 pair

brcs	DC3a		;hours now almost decoded

rjmp	DC3			;repeat loop again


DC3a:

dec		r6		;restore old value of hours
mov		r28,r4	;restore low byte in r28
				;at this point, minutes remainder is in low byte
				; (r29 will be zero)

DC10:

mov		r16,r6		;put hours into r16 for check
cpi		r16,13
brsh	DC9
rjmp	DC8


DC9:

subi	r16,12


DC8:				;flash out time

rcall	IVarClockFlash


DC7:				;decode minutes with MSD double flashed

mov		r16,r28
inc		r7			;use r7 for MSD count
subi	r28,10		;get remaining minutes
brcs	DC7a

rjmp	DC7			;loop back for another subtraction


DC7a:

dec		r7			;restore correct value for MSD
mov		r28,r16		;restore correct remainder (LSDs)

mov		r16,r7		;put in MSD for flashout
sbr		r18,128		;set bit 7(128) for double flash
rcall	IVarClockFlash	;flash out MSD
cbr		r18,128		;clear bit 7(128) of double flash

mov		r16,r28
rcall	IVarClockFlash	;flash out LSDs

sbrs	r18,2		;skip if alarm on flag set, r18,2(4)
rjmp	DCEnd		;if alarm on set, how often interval is flashed out

rcall	IdelayOne		;pause before flashout of how often count
mov		r16,r8
rcall	IFlashNext		;flash out How Often Count


DCEnd:

pop		r29
pop		r28
pop		r16
pop		r6

ret

;---------------------------


IVarClockFlash:		;flashes red or green LED (G=AM R=PM)
					;R16 contains number of flashes
push	r17

rcall	IDelayHalf
rcall	IDelayOne	;delay before flashes start

					;check if AM or PM time (less than 720 = AM)
cpi		r31,2		;go to AM if high byte is <2 
brlo	VCF1		;jump to AM routine
cpi		r31,2
breq	VCF3
rjmp	VCF2		;jump to PM routine


VCF3:

cpi		r30,208		
brlo	VCF1		;jump to AM routine
rjmp	VCF2		;jump to PM routine


VCF1:				; AM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

cbi		PortB,1		;turn on GREEN LED (AM)
rcall	IDelay40th	;on time
sbi		PortB,1		;off Green LED

sbrc	r18,7		;skip if bit 7 is clear - no double flash
rjmp	VCF1ab		;go for double flash


VCF1ac:

rcall	IDelayHalf		;off time
dec		r16
rjmp	VCF1


VCF1ab:			;gives double flash

rcall	IDelay40th
rcall	IDelay40th	;small pause
cbi		PortB,1		;on green LED
rcall	IDelay40th	;on time
sbi		PortB,1		;off Green LED
rjmp	VCF1ac


VCF2:			; PM routine

tst		r16
breq	VCF2a		;clear double flash flag and return

cbi		PortB,0		;turn on RED LED
rcall	IDelay40th	;on time delay
sbi		PortB,0		;turn off RED LED
sbrc	r18,7		;skip if bit 7 is clear - no double flash
rjmp	VCF2ab


VCF2ac:

rcall	IDelayHalf		;off time delay
dec		r16
rjmp	VCF2


VCF2ab:				;give double red flash

rcall	IDelay40th
rcall	IDelay40th	;short off delay
cbi		PortB,0		;on Red LED
rcall	IDelay40th	;on time delay for second flash
sbi		PortB,0		;off Red LED
rjmp	VCF2ac


VCF2a:

pop		r17
cbr		r18,128		;clear bit 7(128) - double flash flag
ret

;-------------------------------------------


SetCompareRegister:

		;puts new value into compare register A.  
		;Default = 218 for high byte, 140 for low byte.
		
push	r1
push	r16

clr		r1
rcall	ThreeGreenFlash

rcall	ButtonStall
rcall	Mult100		;multiply r20 value by 100
mov		r1,r20		;set 100's column

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonStall
rcall	Mult10		;multiply r20 value by 10
add		r1,r20		;add in tens units

rcall	OneGreenFlash	;indicate an entry is needed
rcall	ButtonStall
add		r1,r20		;add in LSD of count -- data is now complete

ldi		r16,218
out 	OCR1AH,r16
out		OCR1AL,r1	;put new compare register A data in place

pop		r16
pop		r1
ret

;-------------------------------------


SecondsCorrection:	;Set seconds of correction and how often (2 entries)

rcall	TwoGreenFlash
rcall	ButtonStall
sts		$61,r20		;seconds of correction (zero to 255)

rcall	OneGreenFlash
rcall	ButtonStall	;enter interval from 0 to 255 - zero = no correction
sts		$62,r20		;put interval into permanent $62 storage
clr		r9			;zero r9
inc		r9			;put one into r9 so correction will 0ccur at next
					;  upcoming midnight. 

ret

;---------------------


SetClock:		;sets clock for AM/PM, hour, minutes, and day-of-week
				;  24 hour time entry required

push	r1		;minutes accumulator
push	r2		;hours accumulator
push	r16
push	r26
push	r27

clr		r26		;low byte accumulator
clr		r27		;high byte accumulator

ldi		r16,4
rcall	SlowGreenFlash	;indicate that four entries are needed

rcall	ButtonStall	;enter MSD of hours 

rcall	Mult10		;multiply value by 10 and store in r2
mov		r2,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonStall		;enter LSD of hours
add		r2,r20			;value in hours now complete in r2

rcall	TwoGreenFlash	;signal another entry needed
rcall	ButtonStall		;enter msd of minutes
rcall	Mult10			;multiply by 10 and store in r1
mov		r1,r20

rcall	OneGreenFlash	;signal another entry needed
rcall	ButtonStall
add		r1,r20			;value in minutes now complete


SetC1:

tst		r2			;total hours count
breq	SetC2		;done
adiw	r26,60
dec		r2
rjmp	SetC1


SetC2:

add		r26,r1		;total minutes count
brcs	SetC3
rjmp	SetEnd


SetC3:

inc		r27			;carry flag was set
rjmp	SetEnd


SetEnd:

bclr	7		;turn off global interrupts

mov		r30,r26		;store low time byte 
mov		r31,r27		;store high time byte

clr		r15			;zero seconds counter

bset	7		;turn on global interrupts

pop		r27
pop		r26
pop		r16
pop		r2
pop		r1

ret

;-------------------------------------


AllValvesOff:		;turns off all valve ports

sbi		PortB,3		;turn off valve bits in PortB (bits 3-7)
sbi		PortB,4		;  valves 1 through 5	
sbi		PortB,5
sbi		PortB,6
sbi		PortB,7
sbi		PortD,6		;(main water valve off)

ret

;--------------------------------


Mult100:		;Multiplies value in R20 by 100 and returns value in R20

push	r1
clr		r1	;zero r1 for upcoming data

tst		r20
breq	Mult100b

ldi		r16,100	;the multiplication value


Mult100a:

add		r1,r16
dec		r20
brne	Mult100a


Mult100b:

mov		r20,r1	;for return of value
pop		r1
ret

;--------------------------------------


Mult10:		; multiplies value in R20 by 10 returning value in R20

push 	r1
clr		r1		;zero R1

tst		R20		;test long count value
breq	Mult10b		;value is zero

ldi		r16,10		;put ten into r16
	

Mult10a:

add		r1,r16		;add ten for each value in R1
dec		r20
brne	Mult10a


Mult10b:

mov		r20,r1		;move r1 value into r16
pop		r1
ret

;--------------------------------


OnSelectedValve:		;R16 has selected valve number

rcall	AllValvesOff

cpi		r16,1
brne	OSV1
cbi		PortB,3		;on bit 3, valve 1
rjmp	OSVEnd


OSV1:

cpi		r16,2
brne	OSV2
cbi		PortB,4		;on bit 4, valve 2
rjmp	OSVEnd


OSV2:

cpi		r16,3
brne	OSV3
cbi		PortB,5		;on bit 5, valve 3
rjmp	OSVEnd


OSV3:

cpi		r16,4
brne	OSV4
cbi		PortB,6		;on bit 6, valve 4
rjmp	OSVEnd


OSV4:

cpi		r16,5
brne	OSVEnd
cbi		PortB,7		;on bit 7, valve 5

ret


OSVEnd:

cbi		PortD,6		;on Main AC Power

ret

;---------------------------------------------------------

;D port check for on

CheckD:

Push	r16

in		r16,PortD
andi	R16,64
brne	CheckD1

sbi		PortB,7
rcall	Delay40th
pop		r16
ret


CheckD1:

cbi		PortB,7
rcall	Delay40th
pop		r16
ret

;--------------------------------

TestStop:

rcall	CheckD
rcall	DelayOne
rcall	DelayOne

ret

;---------------------------------
